searchNodes=[{"ref":"Circuits.SPI.html","title":"Circuits.SPI","type":"module","doc":"This module enables Elixir programs to interact with hardware that&#39;s connected via a SPI bus."},{"ref":"Circuits.SPI.html#bus_names/0","title":"Circuits.SPI.bus_names/0","type":"function","doc":"Return a list of available SPI bus names. If nothing is returned, it&#39;s possible that the kernel driver for that SPI bus is not enabled or the kernel&#39;s device tree is not configured. On Raspbian, run raspi-config and look in the advanced options. iex&gt; Circuits.SPI.bus_names [&quot;spidev0.0&quot;, &quot;spidev0.1&quot;]"},{"ref":"Circuits.SPI.html#close/1","title":"Circuits.SPI.close/1","type":"function","doc":"Release any resources associated with the given file descriptor"},{"ref":"Circuits.SPI.html#info/0","title":"Circuits.SPI.info/0","type":"function","doc":"Return info about the low level SPI interface This may be helpful when debugging SPI issues."},{"ref":"Circuits.SPI.html#open/2","title":"Circuits.SPI.open/2","type":"function","doc":"Open SPI channel On success, returns a reference. Use reference in subsequent calls to transfer SPI bus data Parameters: bus_name is the name of the bus (e.g., &quot;spidev0.0&quot;) opts is a keyword list to configure the bus SPI bus options include: mode: This specifies the clock polarity and phase to use. (0) bits_per_word: bits per word on the bus (8) speed_hz: bus speed (1000000) delay_us: delay between transaction (10)"},{"ref":"Circuits.SPI.html#transfer/2","title":"Circuits.SPI.transfer/2","type":"function","doc":"Perform a SPI transfer. The data should be a binary containing the bytes to send. Since SPI transfers simultaneously send and receive, the return value will be a binary of the same length or an error."},{"ref":"Circuits.SPI.html#t:spi_bus/0","title":"Circuits.SPI.spi_bus/0","type":"type","doc":"SPI bus Call open/2 to obtain an SPI bus reference."},{"ref":"Circuits.SPI.html#t:spi_option/0","title":"Circuits.SPI.spi_option/0","type":"type","doc":"SPI bus options. See open/2."},{"ref":"circuits_spi.html","title":"circuits_spi","type":"module","doc":"Provide an Erlang friendly interface to Circuits Example Erlang code: circuits_spi:open(&quot;spidev0.1&quot;)"},{"ref":"circuits_spi.html#close/1","title":"circuits_spi.close/1","type":"function","doc":"See Circuits.SPI.close/1."},{"ref":"circuits_spi.html#open/1","title":"circuits_spi.open/1","type":"function","doc":"See Circuits.SPI.open/1."},{"ref":"circuits_spi.html#open/2","title":"circuits_spi.open/2","type":"function","doc":"See Circuits.SPI.open/2."},{"ref":"circuits_spi.html#transfer/2","title":"circuits_spi.transfer/2","type":"function","doc":"See Circuits.SPI.transfer/2."},{"ref":"readme.html","title":"Circuits.SPI","type":"extras","doc":"Circuits.SPI Circuits.SPI provides high level abstractions for interfacing to SPI buses on Linux platforms. Internally, it uses the Linux device interface so that it does not require board-dependent code."},{"ref":"readme.html#getting-started","title":"Circuits.SPI - Getting started","type":"extras","doc":"If you&#39;re using Nerves or compiling on a Raspberry Pi or other device with SPI support, then add circuits_spi like any other Elixir library: def deps do [{:circuits_spi, &quot;~&gt; 0.1&quot;}] end Circuits.SPI doesn&#39;t load device drivers, so you&#39;ll need to load any necessary ones beforehand. On the Raspberry Pi, the Adafruit Raspberry Pi SPI instructions may be helpful, (This is already enabled for you if you are using Nerves) A Serial Peripheral Interface (SPI) bus is a common multi-wire bus used to connect components on a circuit board. A clock line drives the timing of sending bits between components. Bits on the Master Out Slave In MOSI line go from the master (usually the processor running Linux) to the slave, and bits on the Master In Slave Out MISO line go the other direction. Bits transfer both directions simultaneously. However, much of the time, the protocol used across the SPI bus has a request followed by a response and in these cases, bits going the &quot;wrong&quot; direction are ignored. This will become more clear in the example below. The following shows an example Analog to Digital Converter (ADC) that reads from either a temperature sensor on CH0 (channel 0) or a potentiometer on CH1 (channel 1). It converts the analog measurements to digital, and sends the digital measurements to SPI pins on the main processor running Linux (e.g. Raspberry Pi). Many processors, like the one on the Raspberry Pi, can&#39;t read analog signals directly, so they need an ADC to convert the signal. The protocol for talking to the ADC in the example below is described in the MCP3002 data sheet. The protocol is very similar to an application program interface (API) for software. It will tell you the position and function of the bits you will send to the ADC, along with how the data (in the form of bits) will be returned. See Figure 6-1 in the data sheet for the communication protocol. Sending a 0x68 first reads the temperature and sending a 0x78 reads the potentiometer. Since the data sheet shows bits, 0x68 corresponds to 01101000b. The leftmost bit is the &quot;Start&quot; bit. The second bit is SGL/DIFF, the third bit is ODD/SIGN, and the fourth bit is MSBF. From table 5-1, if SGL/DIFF==1, ODD/SIGN==0, and MSBF==1 then that specifies channel 0 which is connected to the thermometer. # Make sure that you&#39;ve enabled or loaded the SPI driver or this will # fail. iex&gt; {:ok, ref} = Circuits.SPI.open(&quot;spidev0.0&quot;) {:ok, #Reference&lt;...&gt;} # Read the potentiometer # Use binary pattern matching to pull out the ADC counts (low 10 bits) iex&gt; {:ok, &lt;&lt;_::size(6), counts::size(10)&gt;&gt;} = Circuits.SPI.transfer(ref, &lt;&lt;0x78, 0x00&gt;&gt;) {:ok, &lt;&lt;1, 197&gt;&gt;} iex&gt; counts 453 # Convert counts to volts (1023 = 3.3 V) iex&gt; volts = counts / 1023 * 3.3 1.461290322580645 As shown above, you&#39;ll find out that Elixir&#39;s binary pattern matching is extremely convenient when working with hardware. More information can be found in the Kernel.SpecialForms documentation and by running h &lt;&lt;&gt;&gt; at the IEx prompt."},{"ref":"readme.html#faq","title":"Circuits.SPI - FAQ","type":"extras","doc":"How do I debug? The most common issue is communicating with a SPI device for the first time. First check that a SPI bus is available: iex&gt; Circuits.SPI.bus_names() [&quot;spidev0.0&quot;, &quot;spidev0.1&quot;] If the list is empty, then a SPI bus is either not available, not enabled, or not configured in the kernel. If you&#39;re using Raspbian, run raspi-config and check that SPI is enabled in the advanced options. If you&#39;re on a BeagleBone, try running config-pin and see the Universal I/O project to enable the SPI pins. On other ARM boards, double check that SPI is enabled in the kernel and that the device tree configures it. Where can I get help? Many issues are unrelated to Circuits.SPI. If you expand your searches to include Python and C forums, it&#39;s possible that someone else has run into your problem too. SPI libraries in other languages should be similar to Circuits.SPI so hopefully you&#39;ll find the answer. If that fails, try posting a question to the Elixir Forum. Tag the question with Nerves and it will have a good chance of getting to the right people. Feel free to do this even if you&#39;re not using Nerves. Can I develop code that uses Circuits.SPI on my laptop? You&#39;ll need to fake out the hardware. Code to do this depends on what your hardware actually does, but here&#39;s one example: Compiling and testing Elixir Nerves on your host machine Please share other examples if you have them. Will it run on Arduino? No. This only runs on Linux-based boards. If you&#39;re interested in controlling an Arduino from a computer that can run Elixir, check out nerves_uart for communicating via the Arduino&#39;s serial connection or firmata for communication using the Arduino&#39;s Firmata protocol. How do I call Circuits.SPI from Erlang? An Erlang friendly binding has been provided to simplify syntax when calling Circuits.SPI functions from Erlang code. Instead of prefixing calls with: &#39;Elixir.Circuits.SPI&#39;: you may use the binding: circuits_spi:. For example: circuits_spi:open(&quot;spidev0.1&quot;)."},{"ref":"readme.html#license","title":"Circuits.SPI - License","type":"extras","doc":"Code from the library is licensed under the Apache License, Version 2.0."},{"ref":"porting.html","title":"Porting Elixir/ALE to Circuits.SPI","type":"extras","doc":"Porting Elixir/ALE to Circuits.SPI The Circuits.SPI package is the next version of Elixir/ALE&#39;s SPI support. If you&#39;re currently using Elixir/ALE, you&#39;re encouraged to switch. Here are some benefits: Supported by both the maintainer of Elixir/ALE and a couple others. They&#39;d prefer to support Circuits.SPI issues. Much faster than Elixir/ALE. Simplified API Circuits.SPI uses Erlang&#39;s NIF interface. NIFs have the downside of being able to crash the Erlang VM. Experience with Elixir/ALE has given many of us confidence that this won&#39;t be a problem."},{"ref":"porting.html#code-modifications","title":"Porting Elixir/ALE to Circuits.SPI - Code modifications","type":"extras","doc":"Circuits.SPI is not a GenServer, so if you&#39;ve added ElixirALE.SPI to a supervision tree, you&#39;ll have to take it out and manually call Circuits.SPI.open to obtain a reference. A common pattern is to create a GenServer that is descriptive of what the SPI device does and have it be responsible for all SPI calls. The remain modifications should mostly be mechanical: Rename references to ElixirALE.SPI to Circuits.SPI and elixir_ale to circuits_spi Change calls to ElixirALE.SPI.start_link/2 to Circuits.SPI.open/1. Review the arguments to open to not include any GenServer options. The transfer function now returns {:ok, result} tuples on success so add code to handle that. Consider adding a call to Circuits.SPI.close/1 if there&#39;s an obvious place to release the SPI bus. This is not strictly necessary since the garbage collector will free unreferenced SPI references. Change calls to ElixirALE.SPI.device_names/0 to Circuits.SPI.bus_names/0. If you find that you have to make any other changes, please let us know via an issue or PR so that other users can benefit."}]